#  -*- coding:Utf-8 -*-
# originals ideas from Don & L'In20Cible
# adaptation for OrangeBox games (Windows & Linux servers)
# ./addons/eventscripts/_libs/python/playerviewlib/__init__.py


import es, psyco, langlib, playerlib, weaponlib, vecmath
psyco.full()

esInfo = es.AddonInfo()
esInfo.author = "DoCky"
esInfo.version = "2.106p"
esInfo.name = "PlayerViewLib"
esInfo.basename = "playerview"
esInfo.url = "http://forums.eventscripts.com/viewtopic.php?t=42292 (french)"
esInfo.description = "Allow you to know what entity a player is aiming"

version = es.ServerVar("playerviewlib", "", " Created by DoCky -")
version.set(esInfo.version)
version.makepublic()

# parse "strings.ini"
strings = langlib.Strings(es.getString("eventscripts_gamedir") + "/addons/eventscripts/_libs/python/playerviewlib/strings.ini")
# get language to use (server side)
language = langlib.getLangAbbreviation(es.getString("cl_language"))
language = "en" if language else language



# Internals variables
_queue = []
#_players = {}
_weapon = weaponlib.currentgame.prefix
_functions = "entity", "player", "weapon", "coord"
_cvars = { # convars for ESClassic results
    "userid" : es.ServerVar("playerview_userid", 0, "playerview: userid of player"),
    "function" : es.ServerVar("playerview_function", "", "playerview: name of function executed"),
    "classname" : es.ServerVar("playerview_classname", "", "playerview: name of entity viewved (for entity, player & weapon commands)"),

    "entityid" : es.ServerVar("playerview_entityid", 0, "playerview: index of entity aimed (-1 if none)"),
    "playerid" : es.ServerVar("playerview_playerid", 0, "playerview: userid of player aimed (0 if none)"),
    "playerindex" : es.ServerVar("playerview_playerindex", -1, "playerview: index of player aimed (-1 if none)"),
    "weaponid" : es.ServerVar("playerview_weaponid", 0, "playerview: index of weapon aimed (-1 if none)"),
    "coordinates" : es.ServerVar("playerview_coordinates", "0,0,0", "playerview: coordinates where the player is aiming")}



# get a string from file "strings.ini"
def _language(string, token = ""):
    try: string = strings[string][language]
    except KeyError: string = strings[string]["en"]
    if token: string %= token
    return string



# raise PlayerViewLibError
class PlayerViewLibError(Exception):
    """ PlayerViewLibError Exception """
    def __init__(self, string, token = ""):
        # get string
        self.string = _language(string, token)
    def __str__(self):
        # return error
        return self.string



# handling command (queue system)
class PlayerViewCmd(object):
    """ PlayerViewCmd object """
    def __init__(self):
        self._queue = []

    def __len__(self):
        return len(self._queue)

    def __add__(self, playerview):
        self._queue.append(playerview)

    def __sub__(self, playerview):
        self._queue.remove(playerview)

    def __contain__(self, playerview):
        return playerview in self._queue

    def _command(self):
        # get PlayerView instance
        playerview = self._queue[0]
        # check if function exists
        if not playerview.function in _functions:
            # remove player from queue and raise error
            self - playerview
            raise PlayerViewLibError("function error", playerview.function) 

        # exec internal i_coord method
        if playerview.function == "coord": playerview.i_coord(es.getargv(1))
        # exec internal i_entity method
        else: playerview.i_entity()
        # check if a player is queued
        self.execute()

    def execute(self):
        # if a player is queued, execute command
        if self._queue: self._queue[0].i_execute()

    def clear(self):
        # clear list
        self._queue = []

playerviewcmd = PlayerViewCmd()
# register block for internal command
es.addons.registerBlock("_playerviewlib", "_command", playerviewcmd._command)
# register server command
es.regcmd("_playerview_cmd", "_playerviewlib/_command", "playerview: internal command")



class PlayerView(object):
    """ PlayerView object """
    def __init__(self, userid, callback = None):
        # internal attributes
        self._entities = {}
        self._entityname = ""
        self._filter = ""

        self._userlib = None
        self._playerlib = None
        self._weaponlib = None
        self._vector = None

        # store userid, callback function and block
        self.userid = int(userid)
        self.callback = callback
        self.block = ""

        # public attributes
        self.function = ""
        self.classname = ""
        self.parameters = None

        self.entityid = -1
        self.playerid = 0
        self.playerindex = -1
        self.weaponid = -1
        self.coordinates = "0,0,0"

    """ set convars values """
    def __setitem__(self, cvar, value):
        if _cvars.has_key(cvar): _cvars[cvar].set(value)

    """ get attributes """
    def __getattr__(self, name):
        if name == "coordlist": return map(float, self.coordinates.split(","))
        elif name == "userlib":
            if self._userlib is None: self._userlib = playerlib.getPlayer(self.userid)
            return self._userlib
        elif name == "playerlib":
            if not self.playerid: self._playerlib = None
            elif self._playerlib is None: self._playerlib = playerlib.getPlayer(self.playerid)
            return self._playerlib
        elif name == "weaponlib":
            if self.weaponid == -1: self._weaponlib = None
            elif self._weaponlib is None: self._weaponlib = weaponlib.getWeapon(self.classname)
            return self._weaponlib
        elif name == "vector":
            if str(self._vector) <> self.coordinates: self._vector = vecmath.Vector(self.coordinates)
            return self._vector

    """ execute ESS and PlayerViewLib commands """
    def i_execute(self):
        # if player does not exists
        if not es.exists("userid", self.userid):
            # remove player from queue and raise error
            playerviewcmd - self
            raise PlayerViewLibError("userid error", self.userid)

        # if function is "coord"
        if self.function == "coord":
            # create dynamic prop and exec internal server command
            es.server.insertcmd(
                "es_xprop_dynamic_create %s props_c17/tv_monitor01_screen.mdl;" % self.userid +
                "es _playerview_cmd server_var(eventscripts_lastgive)")
        # else
        else:
            # clear dict
            self._entities.clear()
            # for entities in map
            for index, attribute in es.createentityindexlist().items():
                # store "index" = "classname", "targetname"
                self._entities[index] = attribute["classname"], es.entitygetvalue(index, "targetname")
            # set entity targetname and exec internal server command
            es.server.insertcmd("es_xentsetname %s _pvl_target_;_playerview_cmd" % self.userid)

    """ entity, player & weapon functions """
    def i_entity(self):
        # set default value
        self.classname, entity, found = "", -1, False
        # for index, info in entity list
        for index in self._entities:
            # check if targetname is corresponding
            if es.entitygetvalue(index, "targetname") == "_pvl_target_":
                found, entity = True, index
                self.classname, targetname = self._entities[index]
                # restore last targetname
                es.entitysetvalue(index, "targetname", targetname)
                break
        if not found: entity = -1

        # remove player from queue
        playerviewcmd - self
        # set basic convars
        if self.block is not None:
            self["userid"] = self.userid
            self["function"] = self.function
            self["classname"] = ""

        # if function is "entity"
        if self.function == "entity":
            if self.classname.startswith(self._entityname):
                self.entityid = entity
                if self.block is not None: self["classname"] = self.classname
            else: self.entityid, self.classname = -1, ""

            # call function if "callback" is callable
            if callable(self.callback): self.callback(self)
            # if self.block is not None
            if self.block is not None:
                # set convars first
                self["entityid"] = self.entityid
                # execute block
                es.doblock(self.block)

        # if function is "player"
        elif self.function == "player":
            # get userid of aimed player
            self.playerid = es.getuserid(es.gethandlefromindex(entity))
            # if player exists
            if self.playerid:
                # if filter is given
                if self._filter:
                    # check if player is in list
                    if self.playerid in playerlib.getUseridList(self._filter):
                        # set index
                        self.playerindex = entity
                        if self.block is not None: self["classname"] = self.classname
                    # else, reset values
                    else: self.playerid, self.playerindex, self.classname = 0, -1, ""

                # else, set values
                else:
                    self.playerindex = entity
                    if self.block is not None: self["classname"] = self.classname
            # any player was aimed
            else: self.playerindex, self.classname = -1, ""
            self._playerlib = None

            # call function if "callback" is callable
            if callable(self.callback): self.callback(self)
            # if self.block is not None
            if self.block is not None:
                # set convars first
                self["playerid"] = self.playerid
                self["playerindex"] = self.playerindex
                # execute block
                es.doblock(self.block)

        # if function is "weapon"
        elif self.function == "weapon":
            # if classname does not starts with "weapon_"
            if not self.classname.startswith(_weapon): self.weaponid, self.classname = -1, ""
            elif not self._entityname in self.classname: self.weaponid, self.classname = -1, ""
            else:
                self.weaponid = entity
                if self.block is not None: self["classname"] = self.classname
            self._weaponlib = None

            # call function if "callback" is callable
            if callable(self.callback): self.callback(self)
            # if self.block is not None
            if self.block is not None:
                # set convars first
                self["weaponid"] = self.weaponid
                # execute block
                es.doblock(self.block)

    """ coord function """
    def i_coord(self, index):
        # get coordinates
        self.coordinates = es.getindexprop(index, "CBaseEntity.m_vecOrigin")
        # set new targetname
        es.setentityname(index, "_pvl_model_")
        # remove material
        userid = es.getuserid()
        if userid: es.server.insertcmd("es_xfire %s _pvl_model_ kill " % userid)
        # remove player from queue
        playerviewcmd - self

        # call function with "userid" and "index" parameter
        if callable(self.callback): self.callback(self)
        # if self.block is not None
        if self.block is not None:
            # set convars first
            self["function"] = self.function
            self["userid"] = self.userid
            self["coordinates"] = self.coordinates
            # execute block
            es.doblock(self.block)


    """ Entity Method """
    def entity(self, entity = ""):
        # set infos
        self.function = "entity"
        self._entityname = entity
        # add player in queue
        playerviewcmd + self
        # execute command
        if len(playerviewcmd) == 1: playerviewcmd.execute()

    """ Player Method """
    def player(self, filter = ""):
        self.function = "player"
        self._filter = filter
        playerviewcmd + self
        if len(playerviewcmd) == 1: playerviewcmd.execute()

    """ Weapon Method """
    def weapon(self, weapon = ""):
        self.function = "weapon"
        self._entityname = weapon
        playerviewcmd + self
        if len(playerviewcmd) == 1: playerviewcmd.execute()

    """ Coord Method """
    def coord(self):
        self.function = "coord"
        playerviewcmd + self
        if len(playerviewcmd) == 1: playerviewcmd.execute()



""" Entity Function """
def entity(userid, callback, entity = ""):
    # playerviewlib.entity(<userid>, <callback> [,entity])
    PlayerView(userid, callback).entity(entity)

""" Player Function """
def player(userid, callback, filter = ""):
    # playerviewlib.player(<userid>, <callback> [,filter])
    PlayerView(userid, callback).player(filter)

""" Weapon Function """
def weapon(userid, callback, weapon = ""):
    # playerviewlib.weapon(<userid>, <callback> [,weapon])
    PlayerView(userid, callback).weapon(weapon)

""" Coord Function """
def coord(userid, callback):
    # playerviewlib.coord(<userid>, <callback>)
    PlayerView(userid, callback).coord()



def es_map_start(event_var):
    # clear all player in queue
    playerviewcmd.clear()
# registering es_map_start event
es.addons.registerForEvent(__import__(__name__), "es_map_start", es_map_start)
